################################################################################
#  Replace ... by the location where scripts and datasets are saved            #
################################################################################

filepath=".../"

################################################################################
#  Source/load all scripts, librairies and datasets                            #
################################################################################

library(robustbase)

load(paste(filepath,"Chapter5.rdata",sep=""))

################################################################################
#  Breastfeeding data                                                          #
################################################################################
 

# remove rows with missing values for simpler use later (e.g. step() function)
breastfeed=na.omit(breastfeed)
dim(breastfeed)
# should be 135 x 10.

# DESCRPTIVE ANALYSIS (not in book)
###################################
pairs(breastfeed[,c("educat","age")])
boxplot(breastfeed$age,breastfeed$educat,names=c("age","education"))

summary(breastfeed)
# See data description on page 146.


# CLASSICAL APPROACH (not in book)
##################################
breast.glm=glm(breast~howfedfr+ethnic+educat+age+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,data=breastfeed)
summary(breast.glm)

# Diagnostic
#-----------
par(mfrow=c(3,2))
plot(breast.glm,add.smooth=F)
plot(resid(breast.glm,type="pearson"))
abline(h=0)
breastfeed$age[which(breastfeed$educat==38)]

# some extra diagnostic plots with package boot.
# library("boot")
#glm.diag.plots(breast.glm,iden=TRUE)
# type 0 to quit

# variable selection
#-------------------
anova(breast.glm) # Sequential analysis
step(breast.glm) # based on AIC


# ROBUST APPROACH
#################


#
#       #############
#       # Table 5.4 #
#       #############
#
# Model fitting: Huber's estimator
#---------------------------------
breast.glmrob=glmrob(breast~howfedfr+ethnic+educat+age+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=1.5),data=breastfeed)
# this may produce a warning that can be ignored with some versions of robustbase
#because breast is a 2-level factor
summary(breast.glmrob)

# Model fitting: Mallows's estimator
#-----------------------------------
breast.glmrobWx=glmrob(breast~howfedfr+ethnic+educat+age+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
# this may produce a warning that can be ignored with some versions of robustbase
# because breast is a 2-level factor
summary(breast.glmrobWx)


#
#       ##############
#       # Figure 5.3 #
#       ##############
#

par(mfrow=c(2,1))
plot(breast.glmrobWx$w.r,ylab="w(r)")
identify(breast.glmrobWx$w.r)
# left-clic on observations to identify them, right-clic to end
plot(breast.glmrobWx$w.x,ylab="w(x)")
identify(breast.glmrobWx$w.x)
# left-clic on observations to identify them, right-clic to end

#
#       ###############
#       # Figure 5.4  #
#       ###############
#
par(mfrow=c(1,1))
plot(breastfeed[,"breast"]=="Breast", fitted(breast.glmrobWx),xlab="breast",ylab="Fitted probability of breastfeeding")
text(0.96, 0.02, "90")
text(0.96, 0.10, "63")
text(0.03, 1.02, "75")
text(0.03, 0.98, "11")
text(0.03, 0.95, "115")
text(0.03, 0.91, "14")

#
#       #############
#       # Table 5.5 #
#       #############
#
# ROBUST
#########
# Full model breast.glmrobWx
# linear pred: breast~howfedfr+ethnic+educat+age+pregnancy+howfed+partner+smokenow+smokebf

#submodel without howfedfr
step1howfedfr=glmrob(breast~ethnic+educat+age+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(breast.glmrobWx,step1howfedfr,test="QD")

#submodel without ethnic
step1ethnic=glmrob(breast~howfedfr+educat+age+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(breast.glmrobWx,step1ethnic,test="QD")

#submodel without educat
step1educat=glmrob(breast~howfedfr+ethnic+age+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(breast.glmrobWx,step1educat,test="QD")

#submodel without age
step1age=glmrob(breast~howfedfr+ethnic+educat+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(breast.glmrobWx,step1age,test="QD")

#submodel without pregnancy
step1pregnancy=glmrob(breast~howfedfr+ethnic+educat+age+howfed+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(breast.glmrobWx,step1pregnancy,test="QD")

#submodel without  howfed
step1howfed=glmrob(breast~howfedfr+ethnic+educat+age+pregnancy+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(breast.glmrobWx,step1howfed,test="QD")

#submodel without partner
step1partner=glmrob(breast~howfedfr+ethnic+educat+age+pregnancy+howfed+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(breast.glmrobWx,step1partner,test="QD")

#submodel without smokenow
step1smokenow=glmrob(breast~howfedfr+ethnic+educat+age+pregnancy+howfed+partner+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(breast.glmrobWx,step1smokenow,test="QD")

#submodel without smokebf
step1smokebf=glmrob(breast~howfedfr+ethnic+educat+age+pregnancy+howfed+partner+smokenow,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(breast.glmrobWx,step1smokebf,test="QD")


# CLASSICAL
###########
# Full model breast.glmrobC
# linear pred: breast~howfedfr+ethnic+educat+age+pregnancy+howfed+partner+smokenow+smokebf
breast.glmC=glmrob(breast~howfedfr+ethnic+educat+age+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)

#submodel without howfedfr
step1howfedfrC=glmrob(breast~ethnic+educat+age+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(breast.glmC,step1howfedfrC,test="QD")

#submodel without ethnic
step1ethnicC=glmrob(breast~howfedfr+educat+age+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(breast.glmC,step1ethnicC,test="QD")

#submodel without educat
step1educatC=glmrob(breast~howfedfr+ethnic+age+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(breast.glmC,step1educatC,test="QD")

#submodel without age
step1ageC=glmrob(breast~howfedfr+ethnic+educat+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(breast.glmC,step1ageC,test="QD")

#submodel without pregnancy
step1pregnancyC=glmrob(breast~howfedfr+ethnic+educat+age+howfed+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(breast.glmC,step1pregnancyC,test="QD")

#submodel without  howfed
step1howfedC=glmrob(breast~howfedfr+ethnic+educat+age+pregnancy+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(breast.glmC,step1howfedC,test="QD")

#submodel without partner
step1partnerC=glmrob(breast~howfedfr+ethnic+educat+age+pregnancy+howfed+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(breast.glmC,step1partnerC,test="QD")

#submodel without smokenow
step1smokenowC=glmrob(breast~howfedfr+ethnic+educat+age+pregnancy+howfed+partner+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(breast.glmC,step1smokenowC,test="QD")

#submodel without smokebf
step1smokebfC=glmrob(breast~howfedfr+ethnic+educat+age+pregnancy+howfed+partner+smokenow,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(breast.glmC,step1smokebfC,test="QD")



# COMPLETE STEPWISE PROCEDURE, see text pages 149-150.
# ROBUST
#########

# REMOVE AGE: model step1age
# Linear predictor: howfedfr+ethnic+educat+pregnancy+howfed+partner+smokenow+smokebf

step2howfedfr=glmrob(breast~ethnic+educat+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step1age,step2howfedfr,test="QD")

step2ethnic=glmrob(breast~howfedfr+educat+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step1age,step2ethnic,test="QD")

step2educat=glmrob(breast~howfedfr+ethnic+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step1age,step2educat,test="QD")

step2pregnancy=glmrob(breast~howfedfr+ethnic+educat+howfed+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step1age,step2pregnancy,test="QD")

step2howfed=glmrob(breast~howfedfr+ethnic+educat+pregnancy+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step1age,step2howfed,test="QD")

step2smokenow=glmrob(breast~howfedfr+ethnic+educat+pregnancy+partner+howfed+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step1age,step2smokenow,test="QD")

step2smokebf=glmrob(breast~howfedfr+ethnic+educat+pregnancy+partner+howfed+smokenow,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step1age,step2smokebf,test="QD")

# REMOVE HOWFED: model step2howfed
# Linear predictor: howfedfr+ethnic+educat+pregnancy+partner+smokenow+smokebf

step3howfedfr=glmrob(breast~ethnic+educat+pregnancy+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step2howfed,step3howfedfr,test="QD")

step3ethnic=glmrob(breast~howfedfr+educat+pregnancy+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step2howfed,step3ethnic,test="QD")

step3educat=glmrob(breast~howfedfr+ethnic+pregnancy+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step2howfed,step3educat,test="QD")

step3pregnancy=glmrob(breast~howfedfr+ethnic+educat+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step2howfed,step3pregnancy,test="QD")

step3partner=glmrob(breast~howfedfr+ethnic+educat+pregnancy+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step2howfed,step3partner,test="QD")

step3smokenow=glmrob(breast~howfedfr+ethnic+educat+pregnancy+partner+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step2howfed,step3smokenow,test="QD")

step3smokebf=glmrob(breast~howfedfr+ethnic+educat+pregnancy+partner+smokenow,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step2howfed,step3smokebf,test="QD")

# REMOVE PREGNANCY: model step3pregnancy
# Linear predictor: howfedfr+ethnic+educat+partner+smokenow+smokebf

step4howfedfr=glmrob(breast~ethnic+educat+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step4howfedfr,test="QD")

step4ethnic=glmrob(breast~howfedfr+educat+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step4ethnic,test="QD")

step4educat=glmrob(breast~howfedfr+ethnic+partner+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step4educat,test="QD")

step4partner=glmrob(breast~howfedfr+ethnic+educat+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step4partner,test="QD")

step4smokenow=glmrob(breast~howfedfr+ethnic+educat+partner+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step4smokenow,test="QD")

step4smokebf=glmrob(breast~howfedfr+ethnic+educat+partner+smokenow,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step4smokebf,test="QD")

# REMOVE PARTNER: model step4partner
# Linear predictor: howfedfr+ethnic+educat+smokenow+smokebf

step5howfedfr=glmrob(breast~ethnic+educat+smokenow+smokebf,family=binomial,
weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step5howfedfr,test="QD")

step5ethnic=glmrob(breast~howfedfr+educat+smokenow+smokebf,family=binomial,
weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step5ethnic,test="QD")

step5educat=glmrob(breast~howfedfr+ethnic+smokenow+smokebf,family=binomial,
weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step5educat,test="QD")

step5smokenow=glmrob(breast~howfedfr+ethnic+educat+smokebf,family=binomial,
weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step5smokenow,test="QD")

step5smokebf=glmrob(breast~howfedfr+ethnic+educat+smokenow,family=binomial,
weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step5smokebf,test="QD")

# REMOVE SMOKEBF: model step5smokebf
# Linear predictor: howfedfr+ethnic+educat+smokenow
step6howfedfr=glmrob(breast~ethnic+educat+smokenow,family=binomial,
weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step6howfedfr,test="QD")

step6ethnic=glmrob(breast~howfedfr+educat+smokenow,family=binomial,
weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step6ethnic,test="QD")

step6educat=glmrob(breast~howfedfr+ethnic+smokenow,family=binomial,
weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step6educat,test="QD")

step6smokenow=glmrob(breast~howfedfr+ethnic+educat,family=binomial,
weights.on.x="hat",control=glmrobMqle.control(tcc=1.5),data=breastfeed)
anova(step3pregnancy,step6smokenow,test="QD")

# FINAL MODEL IS: step5smokebf: breast~howfedfr+ethnic+educat+smokenow
summary(step5smokebf)


# CLASSICAL
############

# REMOVE AGE: model step1ageC
# Linear predictor: howfedfr+ethnic+educat+pregnancy+howfed+partner+smokenow+smokebf

step2howfedfrC=glmrob(breast~ethnic+educat+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step1ageC,step2howfedfrC,test="QD")

step2ethnicC=glmrob(breast~howfedfr+educat+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step1ageC,step2ethnicC,test="QD")

step2educatC=glmrob(breast~howfedfr+ethnic+pregnancy+howfed+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step1ageC,step2educatC,test="QD")

step2pregnancyC=glmrob(breast~howfedfr+ethnic+educat+howfed+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step1ageC,step2pregnancyC,test="QD")

step2howfedC=glmrob(breast~howfedfr+ethnic+educat+pregnancy+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step1ageC,step2howfedC,test="QD")

step2smokenowC=glmrob(breast~howfedfr+ethnic+educat+pregnancy+partner+howfed+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step1ageC,step2smokenowC,test="QD")

step2smokebfC=glmrob(breast~howfedfr+ethnic+educat+pregnancy+partner+howfed+smokenow,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step1ageC,step2smokebfC,test="QD")

# REMOVE HOWFED: model step2howfedC
# Linear predictor: howfedfr+ethnic+educat+pregnancy+partner+smokenow+smokebf

step3howfedfrC=glmrob(breast~ethnic+educat+pregnancy+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step2howfedC,step3howfedfrC,test="QD")

step3ethnicC=glmrob(breast~howfedfr+educat+pregnancy+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step2howfedC,step3ethnicC,test="QD")

step3educatC=glmrob(breast~howfedfr+ethnic+pregnancy+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step2howfedC,step3educatC,test="QD")

step3pregnancyC=glmrob(breast~howfedfr+ethnic+educat+partner+smokenow+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step2howfedC,step3pregnancyC,test="QD")

step3partnerC=glmrob(breast~howfedfr+ethnic+educat+pregnancy+smokenow+smokebf,
family=binomial,weights.on.x="hat",control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step2howfedC,step3partnerC,test="QD")

step3smokenowC=glmrob(breast~howfedfr+ethnic+educat+pregnancy+partner+smokebf,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step2howfedC,step3smokenowC,test="QD")

step3smokebfC=glmrob(breast~howfedfr+ethnic+educat+pregnancy+partner+smokenow,
family=binomial,control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step2howfedC,step3smokebfC,test="QD")

# REMOVE EDUCAT: model step3educat
# Linear predictor: howfedfr+ethnic+pregnancy+partner+smokenow+smokebf
step4howfedfrC=glmrob(breast~ethnic+pregnancy+partner+smokenow+smokebf,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step3educat,step4howfedfrC,test="QD")

step4ethnicC=glmrob(breast~howfedfr+pregnancy+partner+smokenow+smokebf,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step3educat,step4ethnicC,test="QD")

step4pregnancyC=glmrob(breast~howfedfr+ethnic+partner+smokenow+smokebf,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step3educat,step4pregnancyC,test="QD")

step4partnerC=glmrob(breast~howfedfr+ethnic+pregnancy+smokenow+smokebf,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step3educat,step4partnerC,test="QD")

step4smokenowC=glmrob(breast~howfedfr+ethnic+pregnancy+partner+smokebf,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step3educat,step4smokenowC,test="QD")

step4smokebfC=glmrob(breast~howfedfr+ethnic+pregnancy+partner+smokenow,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step3educat,step4smokebfC,test="QD")

# REMOVE PREGNANCY: model step4pregnancyC
# Linear predictor: howfedfr+ethnic+partner+smokenow+smokebf
step5howfedfrC=glmrob(breast~ethnic+partner+smokenow+smokebf,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step4pregnancyC,step5howfedfrC,test="QD")

step5ethnicC=glmrob(breast~howfedfr+partner+smokenow+smokebf,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step4pregnancyC,step5ethnicC,test="QD")

step5partnerC=glmrob(breast~howfedfr+ethnic+smokenow+smokebf,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step4pregnancyC,step5partnerC,test="QD")

step5smokenowC=glmrob(breast~howfedfr+ethnic+partner+smokebf,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step4pregnancyC,step5smokenowC,test="QD")

step5smokebfC=glmrob(breast~howfedfr+ethnic+partner+smokenow,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step4pregnancyC,step5smokebfC,test="QD")

# REMOVE SMOKEBF: model step5smokebfC
# Linear predictor: howfedfr+ethnic+partner+smokenow
step6howfedfrC=glmrob(breast~ethnic+partner+smokenow,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step5smokebfC,step6howfedfrC,test="QD")

step6ethnicC=glmrob(breast~howfedfr+partner+smokenow,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step5smokebfC,step6ethnicC,test="QD")

step6partnerC=glmrob(breast~howfedfr+ethnic+smokenow,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step5smokebfC,step6partnerC,test="QD")

step6smokenowC=glmrob(breast~howfedfr+ethnic+partner,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step5smokebfC,step6smokenowC,test="QD")

# REMOVE PARTNER: model step6partnerC
# Linear predictor: howfedfr+ethnic+smokenow
step7howfedfrC=glmrob(breast~ethnic+smokenow,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step5smokebfC,step6howfedfrC,test="QD")

step7ethnicC=glmrob(breast~howfedfr+smokenow,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step5smokebfC,step6ethnicC,test="QD")

step7smokenowC=glmrob(breast~howfedfr+ethnic,family=binomial,
control=glmrobMqle.control(tcc=15),data=breastfeed)
anova(step5smokebfC,step6smokenowC,test="QD")

# Final model:  step6partnerC: breast~howfedfr+ethnic+smokenow



 
#######################
# Doctor visits data  #
#######################
 


#
#       ##############
#       # Figure 5.5 #
#       ##############
#
hist(docvisits$visits,breaks=seq(from=0,to=755,by=5),xlim=c(0,100),xlab="Doctor visits")


#
#       ##############
#       # Figure 5.6 #
#       ##############
#
# CLASSICAL
###########
# Model fitting
docvisits.glm=glm(visits~age+factor(gender)+factor(race)+factor(hispan)+factor(marital)+factor(arthri)+factor(cancer)+factor(hipress)+factor(diabet)+factor(lung)+factor(hearth)+factor(stroke)+factor(psych)+factor(iadla)+factor(adlwa)+edyears+feduc+meduc+log(income+1)+factor(insur), family=poisson,data=docvisits)
summary(docvisits.glm)
# Confidence intervals
docvisitsCI <- cbind(summary(docvisits.glm)$coeff[,1],summary(docvisits.glm)$coeff[,1] -2*summary(docvisits.glm)$coeff[,2], summary(docvisits.glm)$coeff[,1] +2*summary(docvisits.glm)$coeff[,2])

# ROBUST
########
library(robustbase)
# Model fitting: Mallows's estimator
docvisits.glmrobwx=glmrob(visits~age+factor(gender)+factor(race)+factor(hispan)+factor(marital)+factor(arthri)+factor(cancer)+factor(hipress)+factor(diabet)+factor(lung)+factor(hearth)+factor(stroke)+factor(psych)+factor(iadla)+factor(adlwa)+edyears+feduc+meduc+log(income+1)+factor(insur),family=poisson, weights.on.x="hat", data=docvisits, control=glmrobMqle.control(tcc = 1.6))
summary(docvisits.glmrobwx)
# Confidence intervals
docvisitsCIrob <- cbind(summary(docvisits.glmrobwx)$coeff[,1],summary(docvisits.glmrobwx)$coeff[,1] -2*summary(docvisits.glmrobwx)$coeff[,2], summary(docvisits.glmrobwx)$coeff[,1] +2*summary(docvisits.glmrobwx)$coeff[,2])

# Plot 
par(mfrow=c(1,1))
plot(0,0,ylim=c(-0.2,max(docvisitsCI[-1,3],docvisitsCIrob[-1,3])),xlim=c(6,145),type="n",xaxt="n",xlab="",ylab="confidence intervals for the coefficients")
abline(h=0)
abline(v=4,lty=2,lwd=0.3,col="grey")
for(i in 1:24)
  {
    segments(6*i+1, docvisitsCI[i+1,2],6*i+1,docvisitsCI[i+1,3],lwd=2,col="grey")
    points(6*i+0.95,summary(docvisits.glm)$coeff[i+1,1],pch=".",cex=2)
    segments(6*i,docvisitsCI[i+1,2],6*i+2,docvisitsCI[i+1,2],col="grey")
    segments(6*i,docvisitsCI[i+1,3],6*i+2,docvisitsCI[i+1,3],col="grey")

    segments(6*i+2, docvisitsCIrob[i+1,2],6*i+2,docvisitsCIrob[i+1,3],lwd=1)
    points(6*i+1.95,summary(docvisits.glmrobwx)$coeff[i+1,1],pch=".")
    segments(6*i+1,docvisitsCIrob[i+1,2],6*i+3,docvisitsCIrob[i+1,2])
    segments(6*i+1,docvisitsCIrob[i+1,3],6*i+3,docvisitsCIrob[i+1,3])

    abline(v=6*i+4,lty=2,lwd=0.5,col="grey")
   }
    abline(v=6*25+4,lty=2,lwd=0.5,col="grey")
legend(4,0.6,c("Classical","Robust"),lwd=c(2,1),col=c("grey",1))

    text(6+1.5,-0.22,"age",srt=90,cex=0.7,adj=c(0,0))
    text(6*2+1.5,-0.22,"gender",srt=90,cex=0.7,adj=c(0,0))
    text(6*3+1.5,-0.22,"race",srt=90,cex=0.7,adj=c(0,0))
    text(6*4+1.5,-0.22,"hispan",srt=90,cex=0.7,adj=c(0,0))
    text(6*5+1.5,-0.22,"marital",srt=90,cex=0.7,adj=c(0,0))
    text(6*6+1.5,-0.22,"arthri",srt=90,cex=0.7,adj=c(0,0))
    text(6*7+1.5,-0.22,"cancer",srt=90,cex=0.7,adj=c(0,0))
    text(6*8+1.5,-0.22,"hipress",srt=90,cex=0.7,adj=c(0,0))
    text(6*9+1.5,-0.22,"diabet",srt=90,cex=0.7,adj=c(0,0))
    text(6*10+1.5,-0.22,"lung",srt=90,cex=0.7,adj=c(0,0))
    text(6*11+1.5,-0.22,"hearth",srt=90,cex=0.7,adj=c(0,0))
    text(6*12+1.5,-0.22,"stroke",srt=90,cex=0.7,adj=c(0,0))
    text(6*13+1.5,-0.22,"psych",srt=90,cex=0.7,adj=c(0,0))
    text(6*14+1.5,-0.22,"iadla(1)",srt=90,cex=0.7,adj=c(0,0))
    text(6*15+1.5,-0.22,"iadla(2)",srt=90,cex=0.7,adj=c(0,0))
    text(6*16+1.5,-0.22,"iadla(3)",srt=90,cex=0.7,adj=c(0,0))
    text(6*17+1.5,-0.22,"adlwa(1)",srt=90,cex=0.7,adj=c(0,0))
    text(6*18+1.5,-0.22,"adlwa(2)",srt=90,cex=0.7,adj=c(0,0))
    text(6*19+1.5,-0.22,"adlwa(3)",srt=90,cex=0.7,adj=c(0,0))
    text(6*20+1.5,-0.22,"edyears",srt=90,cex=0.7,adj=c(0,0))
    text(6*21+1.5,-0.22,"feduc",srt=90,cex=0.7,adj=c(0,0))
    text(6*22+1.5,-0.22,"meduc",srt=90,cex=0.7,adj=c(0,0))
    text(6*23+1.5,-0.22,"log(income+1)",srt=90,cex=0.7,adj=c(0,0))
    text(6*24+1.5,-0.22,"insur",srt=90,cex=0.7,adj=c(0,0))
    
#
#       ##############
#       # Figure 5.7 #
#       ##############
#
plot(docvisits.glmrobwx$w.r,ylab="w(r)")        
plot(docvisits.glmrobwx$w.x,ylab="w(x)")


# Variable selection, see pages 151-152.
########################################
# without race
docvisits.glmrobwxrace=glmrob(visits~age+factor(gender)+factor(hispan)+factor(marital)+factor(arthri)+factor(cancer)+factor(hipress)+factor(diabet)+factor(lung)+factor(hearth)+factor(stroke)+factor(psych)+factor(iadla)+factor(adlwa)+edyears+feduc+meduc+log(income+1)+factor(insur),family=poisson, weights.on.x="hat", data=docvisits, control=glmrobMqle.control(tcc = 1.6))
anova(docvisits.glmrobwxrace,docvisits.glmrobwx)

## REMOVE race
# additionally remove meduc
docvisits.glmrobwxracemeduc=glmrob(visits~age+factor(gender)+factor(hispan)+factor(marital)+factor(arthri)+factor(cancer)+factor(hipress)+factor(diabet)+factor(lung)+factor(hearth)+factor(stroke)+factor(psych)+factor(iadla)+factor(adlwa)+edyears+feduc+log(income+1)+factor(insur),family=poisson, weights.on.x="hat", data=docvisits, control=glmrobMqle.control(tcc = 1.6))
anova(docvisits.glmrobwxrace,docvisits.glmrobwxracemeduc)

## REMOVE meduc
# additionally remove  log(income+1)

docvisits.glmrobwxracemeducincome=glmrob(visits~age+factor(gender)+factor(hispan)+factor(marital)+factor(arthri)+factor(cancer)+factor(hipress)+factor(diabet)+factor(lung)+factor(hearth)+factor(stroke)+factor(psych)+factor(iadla)+factor(adlwa)+edyears+feduc+factor(insur),family=poisson, weights.on.x="hat", data=docvisits, control=glmrobMqle.control(tcc = 1.6))
anova(docvisits.glmrobwxracemeduc,docvisits.glmrobwxracemeducincome)

## REMOVE income
# additionally remove  insur

docvisits.glmrobwxracemeducincomeinsur=glmrob(visits~age+factor(gender)+factor(hispan)+factor(marital)+factor(arthri)+factor(cancer)+factor(hipress)+factor(diabet)+factor(lung)+factor(hearth)+factor(stroke)+factor(psych)+factor(iadla)+factor(adlwa)+edyears+feduc,family=poisson, weights.on.x="hat", data=docvisits, control=glmrobMqle.control(tcc = 1.6))
anova(docvisits.glmrobwxracemeducincome,docvisits.glmrobwxracemeducincomeinsur)

#
#       #############
#       # Table 5.7 #
#       #############
#
# Final model after variable selection:
# 
finaldocvisits.glmrobwx=glmrob(visits~age+factor(gender)+factor(hispan)+factor(marital)+factor(arthri)+factor(cancer)+factor(hipress)+factor(diabet)+factor(lung)+factor(hearth)+factor(stroke)+factor(psych)+factor(iadla)+factor(adlwa)+edyears+feduc,family=poisson, weights.on.x="hat", data=docvisits, control=glmrobMqle.control(tcc = 1.6))
summary(finaldocvisits.glmrobwx)


#######################
# Costs data          #
#######################



#
#       #############
#       # Table 5.1 #
#       #############
#
costs.glm <- glm(costs~log(los)+adm+ins+age+sex+dest,family=Gamma(link=log),data=costs)
summary(costs.glm)
binf <- summary(costs.glm)$coeff[,1]-qt(0.975,df=93)*summary(costs.glm)$coeff[,2]
bsup <- summary(costs.glm)$coeff[,1]+qt(0.975,df=93)*summary(costs.glm)$coeff[,2]


# Diagnostic
#-----------
#
#       ##############
#       # Figure 5.1 #
#       ##############
#
par(mfrow=c(2,2))
plot(costs.glm,id.n=5,add.smooth=FALSE,cook.levels = c(2, 4))

# some extra diagnostic plots with package boot.
# library("boot")
#glm.diag.plots(costs.glm,iden=TRUE)
# type 0 to quit

#
#       ##############
#       # Figure 5.2 #
#       ##############
#
par(mfrow=c(1,1))
plot(log(costs$los),costs$costs,xlab="log(los)",ylab="cost")
identify(log(costs$los),costs$los)


#
#       #############
#       # Table 5.3 #
#       #############
#
# ROBUST APPROACH
#################

costs.glmrob <- glmrob(costs~log(los)+adm+ins+age+sex+dest,family=Gamma(link=log),data=costs,tcc=1.5)
summary(costs.glmrob)

binfrob <- summary(costs.glmrob)$coeff[,1]-qnorm(0.975)*summary(costs.glmrob)$coeff[,2]
bsuprob <- summary(costs.glmrob)$coeff[,1]+qnorm(0.975)*summary(costs.glmrob)$coeff[,2]

# Variable selection, see pages 144-146
########################################

# ROBUST
#########
# Note: slight numerical differences can appear with respect to the
# numbers on page 145. This is due to small changes in the estimation of the
# nuisance parameter.

costs.glmrobsub1 <- glmrob(costs~log(los)+adm+sex+dest,family=Gamma(link=log),data=costs,tcc=1.5)
anova(costs.glmrobsub1,costs.glmrob)

costs.glmrobsub2 <- glmrob(costs~log(los)+adm+dest,family=Gamma(link=log),data=costs,tcc=1.5)
anova(costs.glmrobsub2,costs.glmrobsub1)

costs.glmrobsub3 <- glmrob(costs~log(los)+adm+sex,family=Gamma(link=log),data=costs,tcc=1.5)
anova(costs.glmrobsub3,costs.glmrobsub1)


# CLASSICAL
##################

costs.glmsub1 <- glm(costs~log(los)+adm+sex+dest,family=Gamma(link=log),data=costs)
anova(costs.glmsub1,costs.glm,test="F")

costs.glmsub2 <- glm(costs~log(los)+adm+dest,family=Gamma(link=log),data=costs)
anova(costs.glmsub2,costs.glmsub1,test="F")

costs.glmsub3 <- glm(costs~log(los)+adm+sex,family=Gamma(link=log),data=costs)
anova(costs.glmsub3,costs.glmsub1,test="F")

